from stats import *
from random import *
from elements import *
from damage import *
from item import *
from effect import *
from util import pos_int_gauss

class CreatureDeath(Exception):
  pass

class TooManyItems(Exception):
  pass

class SlotOccupied(Exception):
  pass

class Creature(object):

  def __init__(self):
    self.name = "default creature"
    self.exp = 1
    self.level = 1
    self.hp = 10
    self.max_hp = 10
    self.mana = 10
    self.max_mana = 10

    self.stats = {
      Strength: 10,
      Defense: 10,
      Agility: 10,
      Evasion: 10,
      Magic: 10,
      Charisma: 10
    }

    self.base_damage = Damage('d2')
    self.target_type = TargetOneFront
    self.wields = None
    self.holds = list()
  
    self.status_effects = []

    self.affinity = None 
    self.skills = []

  def take_damage(self, amount):
    self.hp = self.hp - amount
    if self.hp > self.max_hp:
      self.hp = self.max_hp
    elif self.hp <= 0:
      self.hp = 0
      raise CreatureDeath

  def revive(self, amount=1):
    if self.hp == 0:
      self.take_damage(amount * -1)
 
  def add_effect(self, efct):
    if efct.duration > 0:
      self.status_effects.append(efct)
    efct.start(self) # runs immediate effects as well

  def remove_all_effects(self):
    self.status_effects = list()
 
  def give(self, item):

    from game import game

    # only allow a certain number of items
    if len(self.holds) >= game.policy.creature_hold_slots:
      raise TooManyItems

    # if this is a kind of armor, only one kind allowed
    if item.item_type in armor_types:
      for i in self.holds:
        if i.item_type == item.item_type:
          raise SlotOccupied

    # if nothing else fails, append item
    self.holds.append(item)

  def take(self, item):
    self.holds.remove(item)

  def wield(self, item):
    self.wields = item

  def can_wield(self, item):
    if item.damage is None:
      return False
    for restr, val in item.restrictions.iteritems():
      if self.stats[restr] < val:
        return False
    return True

  def unwield(self):
    """ creature wields nothing, returns what used to be wielded """
    tmp = self.wields
    self.wields = None
    return tmp

  def get_target_type(self):
    if self.wields:
      return self.wields.target_type
    else:
      return self.target_type

  def get_stat(self, key):
    # calculate status effects here, then return

    effects = [i.bonus[key] for i in
      self.holds + self.status_effects + [self.wields]
      if i is not None and i.bonus.has_key(key)]
    return max(self.stats[key] + sum(effects), 0)

  def summary(self):
    ret = "%s Lev: %d\nHP: %d(%d) Mana: %d(%d)\n" % \
      (self.name, self.level, self.hp, self.max_hp, self.mana, self.max_mana)
    ret += "%s(%d) %s(%d) %s(%d) %s(%d) %s(%d) %s(%d)\n" % \
      (Strength.short, self.get_stat(Strength), Defense.short,
      self.get_stat(Defense), Agility.short, self.get_stat(Agility),
      Evasion.short, self.get_stat(Evasion), Magic.short,
      self.get_stat(Magic), Charisma.short, self.get_stat(Charisma))
    return ret

  def short_desc(self):
    return "~6%s~0 Lev %d HP: %d(%d) Mana: %d(%d)" % (self.name, self.level,
      self.hp, self.max_hp, self.mana, self.max_mana)

  def per_turn(self):
    remove_these = list()
    for ef in self.status_effects:
      try:
        ef.per_turn(self)
      except EffectExhausted, e:
        remove_these.append(ef)
    for ef in remove_these:
      self.status_effects.remove(ef)

  def level_up(self):

    from game import game

    self.level += 1

    str_bonus = 1.0
    if (roll('d100') < self.stats[Strength]):
      str_bonus = choice([1.5, 1.5, 1.5, 3])
    self.max_hp = self.max_hp + game.policy.hp_per_level * str_bonus

    mana_bonus = 1.0
    if (roll('d100') < self.stats[Magic]):
      mana_bonus = choice([1.5, 1.5, 1.5, 3])
    self.max_mana = self.max_mana + game.policy.mana_per_level * mana_bonus

    sorted_stats = self.stats.keys()
    sorted_stats.sort(key=self.get_stat)
    sorted_stats.reverse()

    for stkey in sorted_stats:
      self.stats[stkey] = self.stats[stkey] + choice([1, 1, 1, 2])
      if roll('d6') <= 2:
        break

  def choose_target(self, enemy_party=None):

    """ miserable excuse for AI: usually choose enemy with lowest defense """

    pos = list()
    if self.get_target_type() is TargetAll:
      return [c for c in enemy_party.all_members() if c.hp > 0]
    elif self.get_target_type() is TargetFrontRow:
      return [c for c in enemy_party.front if c.hp > 0]
    elif self.get_target_type() is TargetAny or len([c for c in
      enemy_party.front if c.hp > 0]) == 0:
      pos = [c for c in enemy_party.all_members() if c.hp > 0]
    elif self.get_target_type() is TargetOneFront:
      pos = [c for c in enemy_party.front if c.hp > 0]

    if roll('d6') > 3:
      pos.sort(lambda a, b: cmp(a.get_stat(Defense), b.get_stat(Defense)))
      return [pos[0]]
    else:
      return [choice(pos)]

  def __str__(self):
    ret  = "%s Lev: %d Exp: %d HP: %d(%d) Mana: %d(%d)\n" % \
      (self.name, self.level, self.exp, self.hp, self.max_hp, self.mana,
      self.max_mana)
    ret += "Str(%d) Def(%d) Agi(%d) Eva(%d) Mag(%d) Cha(%d)\n" % \
      (self.get_stat(Strength), self.get_stat(Defense), \
      self.get_stat(Agility), self.get_stat(Evasion), \
      self.get_stat(Magic), self.get_stat(Charisma))
    ret += "Elemental affinity: %s\n" % self.affinity
    ret += "Base damage: %s\n" % self.base_damage
    ret += "Target type: %s\n" % self.target_type.short
    ret += "Wields: %s\n" % self.wields
    if len(self.status_effects) > 0:
      ret += "Status effects: %s\n" % ' '.join(i.short for i in
        self.status_effects)
    return ret

def gen_creature(level=1, affinity=None):

  from game import game

  creature = Creature()
  creature.name = game.get_random_creature_name()
  creature.level = level
  creature.hp = creature.max_hp = pos_int_gauss(creature.level * 10,
    (creature.level * 10) / 4)
  creature.mana = creature.max_mana = pos_int_gauss(creature.level * 5,
    (creature.level * 5) / 4)

  # sorry
  if level <= 0:
    level = 1

  if level < len(game.policy.exp_for_level) - 1:
    creature.exp = randrange(game.policy.exp_for_level[level],
      game.policy.exp_for_level[level+1])
  else:
    creature.exp = game.policy.exp_for_level[-1]

  creature.stats[Strength] = 5 + pos_int_gauss(level, level)
  creature.stats[Defense] = 5 + pos_int_gauss(level, level)
  creature.stats[Agility] = 5 + pos_int_gauss(level, level)
  creature.stats[Evasion] = 5 + pos_int_gauss(level, level)
  creature.stats[Magic] = 5 + pos_int_gauss(level, level)
  creature.stats[Charisma] = 5 + pos_int_gauss(level, level)

  # buff some stats randomly
  while roll('d6') <= 3:
    random_stat = choice(creature.stats.keys())
    creature.stats[random_stat] *= 1.5

  creature.base_damage = gen_damage(level)

  if affinity:
    creature.affinity = affinity
  else:
    creature.affinity = choice(game.element_matrix.elements)

  if level < 10:
    creature.target_type = choice([TargetOneFront, TargetOneFront, TargetAny])
  else:
    creature.target_type = choice([TargetOneFront, TargetOneFront,
      TargetAny, TargetAny, TargetFrontRow, TargetAll])

  return creature

if __name__ == '__main__':

  import ui, gamerun
  ui.init()

  import game
  game.game = game.Game()
  game.game.init_test_game()
  from game import game

  print game.element_matrix

  creature = gen_creature()
  armor = gen_armor()
  weapon = gen_weapon()
  potion = gen_potion()
  print armor 
  print potion
  print creature
  creature.give(armor)
  creature.give(potion)
  creature.wield(weapon)
  gamerun.level_up_creature(creature)
  gamerun.level_up_creature(creature)
  gamerun.level_up_creature(creature)
  gamerun.level_up_creature(creature)
  print creature

  creature = gen_creature(49)
  print creature
  creature = gen_creature(50)
  print creature
  creature = gen_creature(51)
  print creature

