from roll import Roller, roll
from random import choice, gauss, randrange

class UnrollableError(Exception):
  """ raised when a Damage object is passed a value that can't be rolled
    (anything that isn't an int or a Roller object) """
  pass

class Damage(object):

  def __init__(self, *dargs):
    """ takes a variable number of arguments specifying Roller objects or
      integers, or a single tuple/list specifying the same """
    self.damages = list()
    if type(dargs[0]) is list or type(dargs[0]) is tuple:
      dargs = dargs[0]
    for d in dargs:
      if type(d) is str:
        self.damages.append(Roller(d))
      elif type(d) is Roller or type(d) is int:
        self.damages.append(d)
      else:
        raise UnrollableError

  def __call__(self):
    sum = 0
    for d in self.damages:
      if type(d) is Roller:
        sum += d()
      else:
        sum += d
    return sum

  def __str__(self):
    return '+'.join([str(i) for i in self.damages])

def gen_damage(level): 

  # kinda weird way of calculating damage for weapon
  dlist = list()
  if level < 10:
    multiplier = ''
    if roll('2d6') == 12:
      multiplier = '2'
    dlist.append('%sd%d' % (multiplier, choice([4, 4, 6, 6, 8, 12])))
  else:
    while True:
      multiplier = ''
      if roll('2d6') < 7:
        multiplier = '2'
      dlist.append('%sd%d' % (multiplier, choice([4, 6, 6, 8, 8, 12, 12])))
      if randrange(200) > level:
        break

  if roll('2d6') > 6:
    next_val = int(gauss(level/2.0, level/2.0))
    if next_val > 0:
      dlist.append(next_val)

  return Damage(dlist)

if __name__ == '__main__':

  # one roller
  testdamage = Damage(Roller('d4'))
  print testdamage
  print testdamage()

  # roller and integer
  testdamage = Damage(Roller('2d6'), +2)
  print testdamage
  print testdamage()

  # string, roller, integer
  testdamage = Damage('d6', Roller('d4'), 4)
  print testdamage
  print testdamage()

  # as list
  testdamage = Damage([Roller('2d4'), 12])
  print testdamage
  print testdamage()

  try:
    testdamage = Damage(Roller('d6'), 4.7)
  except UnrollableError:
    print "caught the error! sweetness"

