from creature import *
from item import *
from roll import roll
from random import choice, randrange
from target_type import *
from party import *
from copy import copy, deepcopy

class Dungeon(object):

  def __init__(self):
    from game import game
    self.levels = [None for i in range(game.policy.deepest_level+1)]
    # leave levels[0] uninitialized
#    self.levels[1] = gen_dungeon_level(1)
    self.deepest_level_reached = 1

  def init_level(self, level):
    self.levels[level] = gen_dungeon_level(level)
    if level > self.deepest_level_reached:
      self.deepest_level_reached = level

class DungeonLevel(object):

  def __init__(self):
    self.affinity = None
    self.possible_creatures = list()
    self.boss = None
    self.relic = None
    self.defeated_boss = False
    self.found_relic = False
    self.parties_defeated = 0

  def __str__(self):
    ret = "This level smells of %s.\n" % self.affinity
    ret += "Creatures that might be encountered on this level:\n"
    for cr in self.possible_creatures:
      ret += str(cr)
    ret += "Boss on this level:\n"
    ret += str(self.boss)
    ret += "Relic on this level:\n"
    ret += str(self.relic)
    return ret

  def gen_enemy_party(self):
    enemy_party = Party()
    for i in range(randrange(1,5)):
      cr = deepcopy(choice(self.possible_creatures))
      if cr.target_type in melee_target_list:
        enemy_party.add_member(cr, position=FrontPosition)
      else:
        enemy_party.add_member(cr, position=BackPosition)

    # move at least one enemy party member to front row
    if len(enemy_party.front) == 0:
      move_me = enemy_party.back[0]
      enemy_party.remove_member(move_me)
      enemy_party.add_member(move_me, position=FrontPosition)

    return enemy_party

  def gen_boss_party(self):
    boss_party = Party()
    boss_party.add_member(deepcopy(self.boss), position=FrontPosition)
    return boss_party

def gen_dungeon_level(level=1):

  from game import game

  lev = DungeonLevel()
  lev.affinity = choice(game.element_matrix.elements)

  creature_gen_count = randrange(-1, 1) + game.policy.creature_types_per_level
  for i in range(creature_gen_count):
    num = roll('d6')
    if num == 1:
      lev.possible_creatures.append(gen_creature(level))
    else:
      lev.possible_creatures.append(gen_creature(level=level,
        affinity=lev.affinity))

  lev.boss = gen_creature(level=level+game.policy.level_boss_bonus,
    affinity=lev.affinity)

  if level == game.policy.deepest_level:
    lev.relic = game.quest_item
  else:
    lev.relic = gen_relic(level)

  return lev

if __name__ == '__main__':
  from game import game
  game.init_test_game()
  level = gen_dungeon_level(1)
  print level
  print level.gen_enemy_party()
  print level.gen_boss_party()

