from stats import *
from random import *
from creature import *
from damage import *
from util import pos_int_gauss
import ui

class EffectExhausted(Exception):
  """ raised when the effect reaches its duration; tells battle engine that
    it's okay to remove the effect from the creature's effect list """
  pass

class Effect(object):

  def __init__(self):
    self.name = 'default effect'
    self.description = 'default effect description'
    self.short = 'Dft'
    self.bonus = dict()
    self.duration = 5
    self.life = 5

  def start(self, target):
    self.life = self.duration
    self.immediate(target)
    return

  def immediate(self, target):
    pass

  def per_turn_action(self, target):
    pass

  # return False to prevent action
  def can_act(self, target):
    return True

  # use this to change who the effected creature can target
  def choose_target(self, chooser, choices):
    pass

  # effects on target per turn
  def per_turn(self, target):
    self.life = self.life - 1
    self.per_turn_action(target)
    if self.life == 0:
      raise EffectExhausted

  def __str__(self):
    duration_str = ''
    if self.duration > 0:
      duration_str = '%d turns' % self.duration
    else:
      duration_str = 'immediate'
    ret = "%s %s (%s) %s\n" % (self.name, self.short, duration_str, self.bonus)
    ret += "Desc: %s\n" % self.description
    return ret

def gen_buff_effect(level=1):
  
  effect = Effect()
  effect.name = choice(['foo', 'bar', 'baz', 'quux'])
  effect.short = effect.name[:3].title()

  effect.duration = pos_int_gauss(5, 2)

  random_stat = choice(stat_list)
  effect.bonus[random_stat] = pos_int_gauss(level*2, level/2)
  effect.description = "%s: %s:%+d for %d turns" % (effect.short,
    random_stat.short, effect.bonus[random_stat], effect.duration)

  return effect

def gen_debuff_effect(level=1):

  effect = Effect()
  effect.name = choice(['foo', 'bar', 'baz', 'quux'])
  effect.short = effect.name[:3].title()
  
  effect.duration = pos_int_gauss(5, 2)

  random_stat = choice(stat_list)
  effect.bonus[random_stat] = pos_int_gauss(level, level/2) * -1
  effect.description = "%s: %s:%+d for %d turns" % (effect.short,
    random_stat.short, effect.bonus[random_stat], effect.duration)

  return effect

class Healer(object):

  def __init__(self, damage):
    self.damage = damage

  def __call__(self, target):
    if target.hp > 0:
      amt = self.damage()
      target.take_damage(amt * -1)
      ui.cwriteln("%s is healed for %d HP!" % (target.name, amt))
    else:
      ui.cwriteln("No effect on %s." % target.name)

def gen_healing_effect(level=1):

  effect = Effect()
  effect.name = 'heal'
  effect.short = effect.name[:3].title()
  effect.duration = 0

  healingfactor = gen_damage(level)
  effect.description = 'Heals target for %s HP.' % healingfactor
  effect.immediate = Healer(healingfactor)
  return effect

class Damager(object):
  def __init__(self, damage):
    self.damage = damage
  def __call__(self, target):
    from creature import CreatureDeath
    amt = self.damage()
    try:
      ui.cwriteln("%s takes %d HP damage!" % (target.name, amt))
      target.take_damage(amt)
    except CreatureDeath:
      ui.cwriteln("%s is killed!" % target.name)

def gen_damage_effect(level=1):

  effect = Effect()
  effect.name = 'damage'
  effect.short = effect.name[:3].title()
  effect.duration = 0

  damagefactor = gen_damage(level)
  effect.description = 'Poison: Does %s HP damage to target.' % damagefactor
  effect.immediate = Damager(damagefactor)
  return effect

class EraseEffects(object):
  def __call__(self, target):
    target.remove_all_effects()
    ui.cwriteln("Status effects on %s have been lifted!" % target.name)

def gen_refresh_effect(level=1):

  effect = Effect()
  effect.name = 'refresh'
  effect.short = effect.name[:3].title()
  effect.description = 'Removes all status effects from target.'
  effect.duration = 0
  effect.immediate = EraseEffects()
  return effect

class Revive(object):
  def __init__(self, damage):
    self.damage = damage
  def __call__(self, target):
    if target.hp > 0:
      ui.cwriteln("%s doesn't need to be revived." % target.name)
    else:
      amt = self.damage()
      target.revive(amt)
      ui.cwriteln("%s is revived! (+%d HP)" % (target.name, amt))

def gen_revive_effect(level=1):

  effect = Effect()
  effect.name = 'revive'
  effect.short = effect.name[:3].title()

  healingfactor = gen_damage(level)
  effect.description = 'Resurrects target, bestowing %s HP.' % healingfactor

  effect.duration = 0
  effect.immediate = Revive(healingfactor)
  return effect

class DoPoison(object):
  def __init__(self, damage):
    self.damage = damage
  def __call__(self, target):
    from creature import CreatureDeath
    amt = self.damage()
    try:
      ui.cwriteln("%s takes %d HP damage from poison!" % (target.name, amt))
      target.take_damage(amt)
    except CreatureDeath:
      ui.cwriteln("%s is killed!" % target.name)

def gen_poison_effect(level=1):

  effect = Effect()
  effect.name = 'poison'
  effect.short = effect.name[:3].title()

  damagefactor = gen_damage(level)
  effect.description = 'Does %s HP damage per turn to target.' % damagefactor
  effect.per_turn_action = DoPoison(damagefactor)
  return effect

if __name__ == '__main__':

  from effect import *

  from game import game
  game.init_test_game()

  creature = gen_creature(5)
  effect = gen_buff_effect(5)
  debuff1 = gen_debuff_effect(8)
  debuff2 = gen_debuff_effect(8)
  refresh_effect = gen_refresh_effect(5)
  revive_effect = gen_revive_effect(5)
  healing_effect = gen_healing_effect(5)
  damage_effect = gen_damage_effect(150)

  print "creature:"
  print creature

  print "effect:"
  print effect

  print "debuff 1:"
  print debuff1

  print "debuff 2:"
  print debuff2

  print "refresh effect:"
  print refresh_effect

  print "healing effect:"
  print healing_effect

  print "damage effect:"
  print damage_effect

  print "revive effect:"
  print revive_effect

  print "added debuff effect 1:"
  creature.add_effect(debuff1)
  print creature

  print "added debuff effect 2:"
  creature.add_effect(debuff2)
  print creature

  print "refreshed:"
  creature.add_effect(refresh_effect)
  print creature

  print "added buff effect:"
  creature.add_effect(effect)
  print creature

  print "damage effect:"
  creature.add_effect(damage_effect)
  print creature

  print "revive effect:"
  creature.add_effect(revive_effect)
  print creature

  print "healing effect:"
  creature.add_effect(healing_effect)
  print creature

  for i in range(8):
    creature.per_turn()
    print creature

