from elements import ElementalMatrix
from policy import GamePolicy
from stats import *
from dungeon import Dungeon
from random import choice
from markov import MarkovChain
import grammar
import re

class Game(object):

  def __init__(self):

    self.element_matrix = None
    self.creatures = list()
    self.items = list()

    self.shop_items = list()

    self.creature_name_pool = list()
    self.relic_name_pool = list()
    self.general_name_pool = list()

    self.policy = GamePolicy()

    self.currency_name_singular = 'smidgeon'
    self.currency_name_plural = 'smidgeons'
    self.currency_name_abbrev = 'sm'

    self.town_name = 'Generic Fantasy Town'
    self.innkeeper_name = 'Alberto'
    self.merchant_name = 'Bethany'
    self.trainer_name = 'Cuervo'
    self.temple_name = 'Efgar'

    self.dungeon_name = 'dungeon'
    self.dungeon = None

    self.quest_item = None
    self.won = False

    self.player_party = None #Party()

  def gen_dungeon_name(self):
    df = open('resources/names/dungeon', 'r')
    names = list()
    for line in df:
      line = line.strip()
      names.append(line)
    df.close()
    return choice(names)

  def gen_currency_names(self):

    cf = open('resources/names/currency', 'r')
    currencies = list()

    for line in cf:
      currencies.append(line.strip())
    currency = choice(currencies)
    (self.currency_name_abbrev, self.currency_name_singular, \
      self.currency_name_plural) = currency.split(',')

    cf.close()

  def gen_relic_name_pool(self):

    nouns = list()
    for line in open('resources/names/nouns'):
      line = line.strip().capitalize()
      nouns.append(line)

    ofs = list()
    for line in open('resources/names/ofs'):
      line = line.strip()
      ofs.append(line)

    used = set()
    i = 0
    while i < 1000:
      next = "%s of %s" % (choice(nouns), choice(ofs))
      if next not in used:
        used.add(next)
        i += 1

    self.relic_name_pool = list(used)

  def gen_general_name_pool(self):

    cfree = grammar.ContextFree(choice(grammar.rulesets), 'Phrase')
    used = set()
    i = 0
    while i < 1000:
      cfree.expand()
      next = re.sub('_', ' ', cfree.get())
      if next not in used and len(next) < 20:
        used.add(next)
        i += 1
     
    self.general_name_pool = list(used) 

  def get_random_relic_name(self):
    new_name = choice(self.relic_name_pool)
    self.relic_name_pool.remove(new_name)
    return new_name

  def get_random_general_name(self):
    new_name = choice(self.general_name_pool)
    self.general_name_pool.remove(new_name)
    return new_name

  def gen_creature_name_pool(self):
 
    markov = MarkovChain(n=4, max=20)
    lines = set()
    for line in open('resources/names/creatures'):
      line = line.strip()
      markov.feed(line)
      lines.add(line)
  
    # generate 1000 names
    generated = set()
    i = 0
    while i < 1000:
      next = markov.gen()
      # make sure this isn't identical to something in the source file,
      # and make sure we haven't already generated this name
      if next not in lines and next not in generated:
        generated.add(next)
        i += 1
  
    self.creature_name_pool = list(generated)
  
  def get_random_creature_name(self):
    new_name = choice(self.creature_name_pool)
    self.creature_name_pool.remove(new_name)
    return new_name

  def init_test_game(self):
    from party import Party
    from item import gen_relic
    self.gen_currency_names()
    self.gen_creature_name_pool()
    self.gen_relic_name_pool()
    self.gen_general_name_pool()
    self.element_matrix = ElementalMatrix()
    self.party = Party()
    self.dungeon_name = self.gen_dungeon_name()
    self.dungeon = Dungeon()
    self.town_name = self.get_random_general_name().title()
    self.innkeeper_name = self.get_random_general_name().title()
    self.merchant_name = self.get_random_general_name().title()
    self.trainer_name = self.get_random_general_name().title()
    self.temple_name = self.get_random_general_name().title()
    self.quest_item = gen_relic(self.policy.deepest_level)

  def gen_game(self):
    pass

if __name__ == '__main__':
  game.gen_currency_names()
  print "singular: %s" % game.currency_name_singular
  print "plural: %s" % game.currency_name_plural
  print "abbrev: %s" % game.currency_name_abbrev

  print "this all takes place in..."
  print game.gen_dungeon_name()
