import ui
import sys
import os
import re
import pickle
from creature import *
from item import *
from party import *
from copy import copy
from math import sqrt

def start_game():

  from game import game
  interpolate = {'town_name': game.town_name,
    'dungeon_name': game.dungeon_name, 'temple_name': game.temple_name,
    'currency_name_plural': game.currency_name_plural,
    'quest_item_name': game.quest_item.name}
  for line in open("resources/text/intro", "r"):
    ui.cwrite(line % interpolate)

  ui.wait_for_keypress() 
  ui.clear_screen()

  for line in open("resources/text/logo", "r"):
    ui.cwrite(line)

  ui.cflush()

  ui.cwriteln("Welcome to ~9LEGEND ~1of the ~9TOMB ~1of ~9FATE!~0")
  ui.cflush()
  while True:
    party_name = ui.input_match_re(re.compile(r"^\w{3,255}"),
      prompt="Enter a name for your party: ",
      errstr="At least three characters, please.")
    sure = ui.confirm("Name your party ~6%s~0 (y/n)? " % party_name)
    if sure:
      break

  game.player_party = Party(name=party_name)

  return select_party_members_menu

def select_party_members_menu():

  from game import game
  import default_members

  while True:

    menu = list()
    ch = 'a'
    for member in default_members.members:
      menu.append([ch, member.summary()])
      ch = chr(ord(ch)+1)

    choices = list()
    while True:
      choices = ui.menu(menu, header="Choose your party members",
        show_char=True, wrap=False, select_multiple=True)
      if len(choices) > 0:
        break

    for i in choices:
      game.player_party.add_member(default_members.members[i])

    names = ui.text_join(["~6%s~0" % m.name for m in
      game.player_party.all_members()])

    ui.cwriteln("You chose %s." % names)

    confirmed = ui.confirm("Is this correct (y/n)? ")
    if confirmed:
      break

  starting_cash = game.policy.starting_cash
  game.player_party.currency += starting_cash
  ui.cwriteln("Your party begins its quest with ~6%d~0 %s." % (starting_cash,
    game.currency_name_plural))

  ui.wait_for_keypress()

  return town_menu

def party_status_screen():
  from game import game

  ui.cflush() 
  ui.cwriteln("Party information")
  ui.hr(ui.cstrlen("Party information")) 
#  ui.cwriteln("~6%s~0 SP: %d(%d) %d%s" % (game.player_party.name,
#    game.player_party.sp, game.player_party.max_sp,
#    game.player_party.currency, game.currency_name_abbrev))
  ui.cwriteln("~6%s~0 %d%s" % (game.player_party.name,
    game.player_party.currency, game.currency_name_abbrev))
  ui.cflush()

  front_row = game.player_party.front
  back_row = game.player_party.back

  ui.cwriteln("Front row:")
  for cr in front_row:
    ui.cwriteln("  %s" % cr.short_desc())
  ui.cwriteln("Back row:")
  for cr in back_row:
    ui.cwriteln("  %s" % cr.short_desc())

  ui.cflush()

  menu = list()
  menu.append(['a', 'Party member status'])
  menu.append(['e', 'Equip party members'])
  menu.append(['i', 'Party inventory'])
  menu.append(['z', 'Change party formation'])
  menu.append(['q', 'Return'])

  choice = ui.menu(menu, header="Select an action", show_char=True,
    wrap=False, select_multiple=False)

  if choice is None:
    ui.cflush()
    return back

  opt = [
    member_status_menu, 
    equip_menu,
    party_inventory_menu,
    party_formation_menu,
    back]

  return opt[choice]

def member_status_menu():
  from game import game

  menu = list()
  ch = 'a'
  for member in game.player_party.all_members():
    menu.append([ch, member.short_desc()])
    ch = chr(ord(ch)+1)

  ui.cflush()
  choice = ui.menu(menu, header="Select a party member", show_char=True,
    wrap=False)

  if choice is None:
    ui.cflush()
    return back
  else:
    return lambda: creature_info(game.player_party.all_members()[choice])

def equip_menu():
  from game import game

  menu = list()
  ch = 'a'
  for member in game.player_party.all_members():
    menu.append([ch, '~6%s~0' % member.name])
    ch = chr(ord(ch)+1)

  ui.cflush()
  choice = ui.menu(menu, header="Equip which party member?", show_char=True)
  if choice is None:
    ui.cflush()
    return back

  pcr = game.player_party.all_members()[choice]

  ui.cflush()
  choice = ui.menu([
    ['p', 'Put on item'],
    ['r', 'Remove item'],
    ['w', 'Wield weapon'],
    ['n', 'Wield nothing']],
    header="Equip action",
    show_char=True)

  if choice is None or choice == 4:
    ui.cflush()
    return back

  opt = [
    put_on_item_menu,
    remove_item_menu,
    wield_menu,
    wield_nothing]

  opt[choice](pcr)

  return back 

def wield_menu(pcr):
  from game import game

  wieldable = [w for w in game.player_party.inventory if pcr.can_wield(w)]
  if len(wieldable) == 0:
    ui.cwriteln("No wieldable items for this party member!")
    ui.wait_for_keypress()
    return

  choice = full_item_menu(src=wieldable, header="Wield which item?")
  if choice is None:
    ui.cflush()
    return

  ui.cwriteln("~6%s~0 now wields the ~6%s~0." % (pcr.name, choice.name))
  ui.wait_for_keypress()

  unwielded = pcr.unwield()
  game.player_party.get_from_inventory(choice)
  pcr.wield(choice)

  if unwielded is not None:
    game.player_party.add_to_inventory(unwielded)

  return

def wield_nothing(pcr):
  from game import game

  if pcr.wields is None:
    ui.cwriteln("%s isn't wielding anything!" % pcr.name)
    ui.wait_for_keypress()
    return
 
  try:
    game.player_party.add_to_inventory(pcr.unwield())
  except TooManyItemsError:
    ui.cwriteln("There is no room in your party's inventory for this weapon.")
    ui.wait_for_keypress()
    return
  ui.cwriteln("%s now wields nothing." % pcr.name)
  ui.wait_for_keypress()
  return

def remove_item_menu(pcr):
  from game import game

  menu = list()
  ch = 'a'
  for i in pcr.holds:
    menu.append([ch, i.list_str()])
    ch = chr(ord(ch)+1)

  ui.cflush()
  choice = ui.menu(menu, header="Remove which item?", show_char=True,
    wrap=False)
  if choice is None:
    ui.cflush()
    return

  picked = pcr.holds[choice]

  try:
    game.player_party.add_to_inventory(picked)
    pcr.take(picked)
  except TooManyItemsError:
    ui.cwriteln("There is no room in your party's inventory for that item.")
    ui.wait_for_keypress()
    return

  ui.cwriteln("%s successfully unequipped." % picked.name)
  ui.wait_for_keypress()
  return

def put_on_item_menu(pcr):
  from game import game

  choice = full_item_menu(src=[item for item in game.player_party.inventory
    if len(item.bonus) > 0], header="Put on which item?")
  if choice is None:
    ui.cflush()
    return

  try:
    pcr.give(choice)
    game.player_party.get_from_inventory(choice)
  except SlotOccupied:
    ui.cwriteln("~6%s~0 is already wearing an item of that type." % \
      pcr.name)
    ui.wait_for_keypress()
    return
  except TooManyItems:
    ui.cwriteln("~6%s~0 can't carry any more items." % pcr.name)
    ui.wait_for_keypress()
    return

  ui.cwriteln("~6%s~0 now wears the ~6%s~0." % (pcr.name, choice.name))
  ui.wait_for_keypress()
  return back

def party_formation_menu():
  from game import game
  ui.cwriteln("party formation menu")

  ch = 'a'
  menu = list() 
  for cr in game.player_party.all_members():
    menu.append([ch, cr.name])
    ch = chr(ord(ch)+1)

  ui.cflush()
  while True: 
    choices = ui.menu(menu, header="Select members for front row",
      show_char=True, select_multiple=True)
    if choices is None:
      return back
    if len(choices) == 0:
      ui.cwriteln("At least one party member must be assigned " + 
        "to the front row.")
      ui.wait_for_keypress()
      continue
    else:
      to_front = [game.player_party.all_members()[i] for i in choices]
      mempos = list()
      for cr in game.player_party.all_members():
        if cr in to_front:
          mempos.append([cr, FrontPosition])
        else:
          mempos.append([cr, BackPosition])
      game.player_party.assign_positions(mempos)
      ui.cwriteln("Success!")
      ui.wait_for_keypress()
      break

  return back

def party_inventory_menu():
  from game import game
  picked = full_item_menu(src=game.player_party.inventory,
    header="Select an item for details")
  return back

def creature_info(cr):
  from game import game
  ui.cwriteln(cr.short_desc())
  for stat in stat_list:
    ui.cwrite("%s(~6%d~0/~6%d~0) " % (stat.short, cr.get_stat(stat),
      cr.stats[stat]))
  ui.cflush()
  try:
    ui.cwriteln("Exp: ~6%d~0 (~6%d~0 needed for next)" % (cr.exp,
      game.policy.exp_for_level[cr.level+1]))
  except IndexError:
    ui.cwriteln("Exp: ~6%d~0 (highest level possible)" % cr.exp)
  ui.cflush()
  if cr.wields:
    ui.cwriteln("~6Wielding~0")
    ui.hr(len("Wielding"))
    ui.cwriteln("%s" % cr.wields.list_str())
  if len(cr.holds) > 0:
    ui.cflush()
    ui.cwriteln("~6Wearing~0")
    ui.hr(len("Wearing"))
    for item in cr.holds:
      ui.cwriteln("%s" % item.list_str())
  ui.wait_for_keypress()
  ui.cflush()
  return back

def town_menu():
  from game import game

  ui.cflush() 
  choice = ui.menu([
    ['a', "%s's Inn" % game.innkeeper_name],
    ['b', "%s's General Store" % game.merchant_name],
    ['c', "Temple of %s" % game.temple_name],
    ['d', "Descend into the %s" % game.dungeon_name],
    ['z', "Party status"],
    ['q', "Quit"]],
    header="Town of %s" % game.town_name,
    show_char=True, wrap=False)

  if choice is None:
    ui.cflush() 
    return quit

  opt = [
    inn_menu,
    store_menu,
    temple_menu,
    dungeon_menu,
    party_status_screen,
    quit]

  return opt[choice]

def inn_menu():
  from game import game

  ui.cflush()
  choice = ui.menu([
    ['s', "Stay for the night"],
    ['q', "Return to town"]],
    header="%s's Inn" % game.innkeeper_name,
    show_char=True)

  if choice is None:
    ui.cflush()
    return town_menu

  opt = [
    sleep_action,
    town_menu]

  return opt[choice]

def sleep_action():
  from game import game

  lev_total = sum([cr.level for cr in game.player_party.all_members()])
  lev_avg = lev_total / float(len(game.player_party.all_members()))
  cost = int(lev_avg * game.policy.level_inn_multiplier)

  okay = ui.confirm("That will cost ~6%d~0 %s.  Okay (y/n)? " % (cost,
    game.currency_name_abbrev))
  if okay:
    if game.player_party.currency >= cost:
      game.player_party.currency -= cost
      ui.cwriteln("Your party sleeps soundly and feels refreshed.")
      for cr in game.player_party.all_members():
        cr.remove_all_effects()
        if cr.hp > 0: # don't heal dead people
          cr.hp = cr.max_hp
          cr.mana = cr.max_mana
      ui.wait_for_keypress()
    else:
      ui.cwriteln("You can't afford that!")
      ui.wait_for_keypress()

  return inn_menu

def store_menu():
  from game import game

  ui.cflush()
  choice = ui.menu([
    ['b', "Buy items"],
    ['s', "Sell items"],
    ['c', "Commission a new item"],
    ['z', "Party status"],
    ['q', "Return to town"]],
    header="%s's General Store" % game.merchant_name,
    prompt="Your party has %d %s." % (game.player_party.currency,
      game.currency_name_abbrev),
    show_char=True)

  if choice is None:
    ui.cflush()
    return town_menu

  opt = [
    buy_item_menu,
    sell_item_menu,
    commission_item_action,
    party_status_screen,
    town_menu]

  return opt[choice]

def sell_item_menu():
  from game import game

  if len(game.player_party.inventory) == 0:
    ui.cwriteln("You don't have any items to sell!")
    ui.wait_for_keypress()
    return back

  picked = full_item_menu(game.player_party.inventory,
    header="Sell which item?")

  if picked is None:
    ui.cwriteln("Cancelled.")
    ui.wait_for_keypress()
    return store_menu
  else:
    buybackval = int(picked.value * 0.333)
    okay = ui.confirm("We'll pay ~6%d~0 %s for the ~6%s~0. Okay (y/n)? " % \
      (buybackval, game.currency_name_abbrev, picked.name))
    if okay:
      game.player_party.get_from_inventory(picked)
      game.player_party.currency += buybackval
  return store_menu

def buy_item_menu():
  from game import game

  if len(game.shop_items) == 0:
    ui.cwriteln("No items are on sale! You should commission an item.")
    ui.wait_for_keypress()
    return back

  picked = full_item_menu(game.shop_items)

  if picked is None:
    ui.cwriteln("Cancelled.")
    ui.wait_for_keypress()
    return back
  else:
    charismas = [c.get_stat(Charisma) for c in game.player_party.all_members()]
    charismas.reverse()
    highest_charisma = min(50, charismas[0])
    cdiscount = int(picked.value * ((charismas[0]/50.0) *
      game.policy.charisma_discount_factor))
    cost = picked.value - cdiscount
    okay = ui.confirm("That will cost %d %s.  Pay (y/n)? " % (cost,
      game.currency_name_abbrev))
    if okay:
      if game.player_party.currency > cost:
        try:
          game.player_party.add_to_inventory(copy(picked))
        except TooManyItemsError:
          ui.cwriteln("Your party can't hold any more items!")
          ui.wait_for_keypress()
          return back
        game.player_party.currency -= cost
        ui.cwriteln("You bought the ~6%s~0." % picked.name)
        ui.wait_for_keypress()
      else:
        ui.cwriteln("You can't afford that!")
        ui.wait_for_keypress()

  return back

def full_item_menu(src=None, header="Select an item", display_only=False,
  show_char=True):
  from game import game

  """ paged menu for items """

  begin_index = 0
  more_len = 10
  page_no = 1

  while True:
    menu = list()
    ch = 'a'
    last_index = min(begin_index+more_len, len(src))
    for i in range(begin_index, last_index):
      item = src[i]
      menu.append([ch, item.list_str()])
      ch = chr(ord(ch)+1)
    prompt = "[Esc] cancels"
    if last_index != len(src):
      prompt = "[Space] for next page, [Esc] cancels"
    choice = ui.menu(menu, header="%s (page %d)" % (header, page_no),
      wrap=False, show_char=show_char, prompt=prompt,
      display_only=display_only)
    if choice is not None:
      return src[begin_index + choice]
    elif last_index == len(src):
      return None
    else:
      begin_index += more_len
      page_no += 1
      ui.cflush()

def commission_item_action():
  from game import game

  levelstr = ui.input_match_re(re.compile(r"^\d+$"),
    prompt="Commission an item at what level? ",
    errstr="Numbers only, please.")

  if levelstr is None:
    print "Cancelled."
    ui.wait_for_keypress()
    return store_menu

  level = int(levelstr)
  cost = 0
  if game.won:
    cost = 5
  else:
    cost = level * game.policy.level_commission_cost_multiplier

  okay = ui.confirm(prompt="That will cost %d %s.  Okay (y/n)? " % (cost,
    game.currency_name_abbrev))

  if okay:
    if game.player_party.currency < cost:
      ui.cwriteln("You don't have enough %s!" % game.currency_name_plural)
      ui.wait_for_keypress()
    else:
      game.player_party.currency -= cost
      print "Your item:"
      new_item = choice([gen_weapon, gen_armor, gen_potion])(level)
      ui.cwriteln(new_item.list_str())
      okay2 = ui.confirm(prompt="Add this item to the shop (y/n)? ")
      if okay2:
        ui.cwriteln("Added.")
        ui.wait_for_keypress()
        game.shop_items.append(new_item)
      else:
        ui.cwriteln("You abandoned your commission.")
        ui.wait_for_keypress()
  else:
    ui.cwriteln("Cancelled.")
    ui.wait_for_keypress()
    
  return store_menu   

def dojo_menu():
  print "dojo placeholder"
  return town_menu

def temple_menu():
  from game import game
 
  ui.cflush() 
  choice = ui.menu([
    ['a', 'Resurrect a party member'],
    ['b', 'Ask about the nature of this world'],
    ['r', 'Sell a relic'],
    ['i', 'Relic information'],
    ['s', 'Save your game'],
    ['q', 'Return to town']],
    header="Temple",
    show_char=True)

  if choice is None:
    ui.cflush()
    return town_menu

  opt = [
    resurrect_menu,
    elemental_display,
    relic_sale,
    relic_info_screen,
    save_menu,
    town_menu]

  return opt[choice]

def relic_info_screen():
  from game import game

  ui.cwriteln("Our temple seeks the ~6%s~0." % game.quest_item.name)
  ui.cwriteln("   It is said to be found on level ~6%d~0 of the ~6%s~0." % \
    (game.policy.deepest_level, game.dungeon_name))
  ui.cflush()
  ui.cwriteln("So far, you have found the following relics:")
  ui.wait_for_keypress()

  choice = full_item_menu([lev.relic for lev in game.dungeon.levels \
    if lev is not None and lev.found_relic is True],
    header="Relics you've found", show_char=False,
    display_only=True)

  ui.wait_for_keypress()

  return temple_menu

def save_menu():
  do_save()
  return temple_menu

def resurrect_menu():
  from game import game

  revivable = [cr for cr in game.player_party.all_members() if cr.hp == 0]
  if len(revivable) == 0:
    ui.cwriteln("No one in your party needs resurrection.")
    ui.wait_for_keypress()
    return temple_menu

  ch = 'a'
  menu = list()
  for cr in revivable:
    menu.append([ch, cr.name])
    ch = chr(ord(ch)+1)

  ui.cflush()
  choice = ui.menu(menu, header="Resurrect which party member?",
    show_char=True)
 
  if choice is None:
    ui.cflush()
    return temple_menu

  picked = revivable[choice]
  cost = picked.level * game.policy.level_resurrect_multiplier

  okay = ui.confirm("Resurrecting ~6%s~0 will cost ~6%d~0 %s. Okay (y/n)? " % \
    (picked.name, cost, game.currency_name_abbrev))

  if okay:
    if game.player_party.currency >= cost:
      game.player_party.currency -= cost
      picked.revive()
      picked.hp = picked.max_hp
      ui.cwriteln("%s has been resurrected to full health." % picked.name)
      ui.wait_for_keypress()
    else:
      ui.cwriteln("You can't afford that!")
      ui.wait_for_keypress()
 
  return temple_menu

def elemental_display():
  from game import game

  element_names = ["~6%s~0" % n for n in game.element_matrix.elements]
  ui.cwriteln("The elements comprising this world are %s." % \
    ui.text_join(element_names))

  for attack in game.element_matrix.matrix:
    strong_against = list()
    weak_against = list()
    for defense in game.element_matrix.matrix:
      if game.element_matrix.matrix[attack][defense] > 1:
        strong_against.append(defense)
      elif game.element_matrix.matrix[attack][defense] < 1:
        weak_against.append(defense)
    strong_names = ["~6%s~0" % n for n in strong_against]
    weak_names = ["~6%s~0" % n for n in weak_against]
    if len(strong_against) > 0 and len(weak_against) > 0:
      ui.cwriteln("~6%s~0 is strong against %s,\n\tbut weak against %s." % \
        (attack.title(), ui.text_join(strong_names), ui.text_join(weak_names)))
    elif len(strong_against) > 0:
      ui.cwriteln("~6%s~0 is strong against %s." % (attack.title(), \
        ui.text_join(strong_names)))
    elif len(weak_against) > 0:
      ui.cwriteln("~6%s~0 is weak against %s." % (attack.title(), \
        ui.text_join(weak_names)))

  ui.cflush()
  ui.wait_for_keypress()

  return back

def relic_sale():
  from game import game

  choice = full_item_menu(\
    src=[r for r in game.player_party.inventory if r.item_type is RelicItem],
    header="Sell which relic?")

  if choice is None:
    ui.cwriteln("Cancelled.")
    ui.wait_for_keypress()
    ui.cflush()
    return temple_menu

  okay = ui.confirm("Sell the %s to the temple for %d %s (y/n)? " % \
    (choice.name, choice.value, game.currency_name_abbrev))

  if okay:
    game.player_party.get_from_inventory(choice)
    game.player_party.currency += choice.value
    if choice is game.quest_item:
      ui.cwriteln("You have recovered the ~6%s~0!  ~EYou win!~0" % \
        choice.name)
      ui.cwriteln("(As a reward for your bravery, ~6commissions at the " + \
        "general store have\nbeen drastically discounted~0.)")
      ui.wait_for_keypress()
      game.won = True
      return temple_menu
    else:
      ui.cwriteln("The temple thanks you for retrieving the relic.")
      ui.wait_for_keypress()

  return temple_menu

def dungeon_menu():
  from game import game

  if game.dungeon.deepest_level_reached == 1:
    return lambda: dungeon_level(1)

  level = 0
  while True:
    level = int(ui.input_match_re(re.compile(r"^-?\d{1,3}$"),
      prompt="Descend to which level of the %s (1 to %d)? " % \
      (game.dungeon_name, game.dungeon.deepest_level_reached),
      errstr="Please enter a number."))
    if level < 1:
      ui.cwriteln("Sorry, no elemental planes in this game!")
    elif level > game.dungeon.deepest_level_reached:
      ui.cwriteln("You currently cannot descend past level %d." % \
        game.dungeon.deepest_level_reached)
    else:
      break

  return lambda: dungeon_level(level)

def dungeon_level(level):
  from game import game

  if game.dungeon.levels[level] is None:
    game.dungeon.init_level(level)

  menu = list()
  menu.append(['s', 'Search this level'])
  menu.append(['z', 'Party status'])
  menu.append(['q', 'Ascend to town'])

  ui.cflush()
  choice = ui.menu(menu, header="Deep within the %s (Level: %d)" % \
    (game.dungeon_name, level), show_char=True)

  opt =[
    lambda: search_level(level),
    party_status_screen,
    ascend_to_town_screen]

  if choice is None:
    ui.cflush()
    return ascend_to_town_screen

  return opt[choice]

def search_level(level):
  from game import game
  # chance of finding treasure, chance of encountering enemy
  currlev = game.dungeon.levels[level]
  rval = roll('d8')
  if rval == 1:
    enemy_party = currlev.gen_enemy_party()
    ui.cwriteln("You encounter an enemy party!")
    if do_battle(enemy_party) is True:
      currlev.parties_defeated += 1
  elif rval == 2: # and roll('d4') <= len(game.player_party.all_members()):
    val = game.policy.level_treasure_multiplier * level
    val = pos_int_gauss(val, val/2)
    ui.cwriteln("You found ~6%d~0 %s!" % (val, game.currency_name_plural))
    ui.wait_for_keypress()
    game.player_party.currency += val
  elif rval == 3 and roll('d6') == 1 and currlev.found_relic is False:
    ui.cwriteln("You found this level's relic!\n\n%s" % \
      currlev.relic.list_str())
    game.player_party.add_to_inventory(currlev.relic)
    currlev.found_relic = True
    ui.wait_for_keypress()
  elif rval == 4:
    if currlev.defeated_boss is False and \
      roll('d20') < currlev.parties_defeated:
      ui.cwriteln("~9BOSS ENCOUNTER!~0")
      boss_party = currlev.gen_boss_party()
      if do_battle(boss_party):
        currlev.defeated_boss = True
        if level + 1 < len(game.dungeon.levels):
          ui.cflush()
          ui.cwriteln("You can now descend to level %d!" % (int(level) + 1))
          game.dungeon.init_level(int(level) + 1)
    else:
      ui.cwriteln("You find nothing, but the dark hallways beckon...")
      ui.cflush()
  else:
    ui.cwriteln("You find nothing, but the dark hallways beckon...")
    ui.cflush()
  return back

def do_battle(enemy_party):
  from game import game

  enemies_defeated = list()
  battle_ended = False

  while True:

    # print status screen
    ui.wait_for_keypress()
    ui.clear_screen()
    ui.cwriteln("Battle!")
    ui.hr(len("Battle!"))
    ui.cflush()
    ui.cwriteln("Enemy front row")
    for cr in enemy_party.front:
      ui.cwriteln("  %s" % cr.short_desc())
    if len(enemy_party.back) > 0:
      ui.cwriteln("Enemy back row")
      for cr in enemy_party.back:
        ui.cwriteln("  %s" % cr.short_desc())
    ui.cflush()
    ui.cwriteln("Your front row")
    for cr in game.player_party.front:
      ui.cwriteln("  %s" % cr.short_desc())
    if len(game.player_party.back) > 0:
      ui.cwriteln("Your back row")
      for cr in game.player_party.back:
        ui.cwriteln("  %s" % cr.short_desc())
    ui.cflush()
    
    everyone = enemy_party.all_members() + game.player_party.all_members()
    everyone.sort(lambda a, b: cmp(b.get_stat(Agility), a.get_stat(Agility)))

    for cr in everyone:

      if cr.hp <= 0:
        continue

      if cr in game.player_party.all_members():
        while True:
          ui.cflush()
          choice = ui.menu([
            ['a', 'Attack'],
            ['b', 'Pass'],
            ['i', 'Use item'],
            ['x', 'Examine enemy'],
            ['q', 'Attempt escape']],
            header="Command for %s HP: %d(%d)" % (cr.name, cr.hp, cr.max_hp),
            show_char=True, prompt="")

          if choice == 0: # attack
            targeted = choose_target(enemy_party=enemy_party,
              ttype=cr.get_target_type(), desc="attack")
            if targeted is None:
              continue
            ui.cwriteln("You targeted: %s" % ', '.join([c.name.capitalize()
              for c in targeted]))
            do_attack(attacker=cr, targets=targeted)
            break
          elif choice == 1:
            ui.cwriteln("~6%s~0 passes." % cr.name)
            break
          elif choice == 2:
            ret = do_use_item(attacker=cr, attacker_party=game.player_party,
              enemy_party=enemy_party)
            if ret is None: # cancelled, or no items available
              continue
            else:
              break
          elif choice == 3:
            targeted = choose_target(enemy_party=enemy_party,
              ttype=TargetAny, desc="examination")
            if targeted is None:
              continue
            else:
              ui.cwriteln("%s" % str(targeted[0]))
              ui.wait_for_keypress()
              break
          elif choice == 4 or choice is None:
            ui.cflush()
            okay = ui.confirm("Run from this battle (y/n)? ")
            if okay:
              ui.cwriteln("Your party escapes!")
              ui.wait_for_keypress()
              return None
            else:
              ui.cwriteln("Cancelled.")
              continue

      else:
        targeted = cr.choose_target(enemy_party=game.player_party)
        do_attack(attacker=cr, targets=targeted)
      if game.player_party.dead():
        ui.cwriteln("Your party died...")
        ui.wait_for_keypress()
        party_death()

      cr.per_turn() # runs status effects

      enemies_defeated += enemy_party.cull()
      if len(enemy_party.all_members()) == 0:
        battle_ended = True
        break

    if battle_ended:
      break

  ui.cwriteln("Your party was victorious!")
  ui.wait_for_keypress()

  exp_total = int(sum([1.75 * sqrt(en.exp) for en in enemies_defeated]))
  if exp_total <= 0:
    exp_total = 1
  survivors = [cr for cr in game.player_party.all_members() if cr.hp > 0]
  for cr in survivors:
    cr.exp += exp_total
    ui.cwriteln("~6%s~0 gets %d experience point%s!" % (cr.name, exp_total,
      (exp_total > 1 and 's' or '')))
    # don't level up beyond maximum
    if cr.level >= len(game.policy.exp_for_level):
      continue
    while cr.exp > game.policy.exp_for_level[cr.level+1]:
      level_up_creature(cr)

  return True

def do_use_item(attacker, attacker_party, enemy_party):
  from game import game

  usable = [i for i in attacker_party.inventory if len(i.use_effects) > 0]
  if len(usable) == 0:
    ui.cwriteln("You don't have any usable items!")
    ui.wait_for_keypress()
    return None
    
  picked = full_item_menu(src=usable, header="Use which item?")
  if picked is None:
    ui.cwriteln("Cancelled.")
    ui.wait_for_keypress()
    return None

  if picked.target_type is TargetFriendly:
    pickparty = attacker_party
  else:
    pickparty = enemy_party
 
  targeted = choose_target(enemy_party=enemy_party,
    ttype=picked.target_type, desc="item")
  if targeted is None:
    return None

  tstr = ui.text_join([t.name for t in targeted])
  ui.cwriteln("%s uses the ~6%s~0 on %s!" % (attacker.name, picked.name,
    tstr))
  for cr in targeted:
    for ef in picked.use_effects:
      cr.add_effect(ef)

  attacker_party.get_from_inventory(picked)
  return True

def do_attack(attacker=None, targets=[]):
  from game import game

  damage = None
  aaff = None # attacker's elemental affinity
  if attacker.wields is not None:
    damage = attacker.wields.damage
    aaff = attacker.wields.affinity
  else:
    damage = attacker.base_damage
    aaff = attacker.affinity

  for t in targets:

    # determine target's elemental affinity
    taff = t.affinity
    for i in t.holds:
      if i is None:
        continue
      if i.item_type is BodyArmorItem:
        taff = i.affinity
    
    damage_bonus = attacker.get_stat(Strength) / float(t.get_stat(Defense))
    chance_to_hit = attacker.get_stat(Agility) / float(t.get_stat(Evasion))
    if damage_bonus > 2:
      damage_bonus = 2
    elif damage_bonus < 0.5:
      damage_bonus = 0.5

    if chance_to_hit > 0.917:
      chance_to_hit = 0.917
    elif chance_to_hit < 0.083:
      chance_to_hit = 0.083

    elemental_bonus = 1
    if aaff is not None and taff is not None:
      elemental_bonus = game.element_matrix.matrix[aaff][taff]

    if roll('d100') < (chance_to_hit * 100):
      damage_value = int(damage() * damage_bonus * elemental_bonus)
      ui.cwriteln("~6%s~0 hits ~6%s~0 for ~9%d~0 HP!" % \
        (attacker.name.capitalize(), t.name, damage_value))
      if elemental_bonus > 1:
        ui.cwriteln("~1-~3-~B> ~0Elemental ~Ebonus~0! (%s vs %s)" % \
          (aaff, taff))
      elif elemental_bonus < 1:
        ui.cwriteln("~1-~3-~B> ~0Elemental ~9weakness~0! (%s vs %s)" % \
          (aaff, taff))
      try:
        t.take_damage(damage_value)
      except CreatureDeath:
        if t in game.player_party.all_members():
          ui.cwriteln("~9%s is dead!~0" % t.name)
        else:
          ui.cwriteln("~E%s is dead!~0" % t.name.capitalize())
    else:
      ui.cwriteln("~E%s evades %s's attack!~0" % (t.name.capitalize(),
        attacker.name))

  return True

def choose_target(enemy_party=None, ttype=None, desc="action"):
  from game import game

  targeted = list()

  if ttype is TargetAll:
    targeted = enemy_party.all_members()
  elif ttype is TargetFrontRow:
    targeted = enemy_party.front
  elif ttype in (TargetAny, TargetOneFront, TargetFriendly):
    menu = list()
    src = list()
    if ttype is TargetAny:
      src = enemy_party.all_members()
    elif ttype is TargetOneFront:
      src = enemy_party.front
    elif ttype is TargetFriendly:
      src = game.player_party.all_members()

    ch = 'a'
    for tg in src:
      menu.append([ch, "%s HP: %d(%d)" % (tg.name.capitalize(), tg.hp,
        tg.max_hp)])
      ch = chr(ord(ch)+1)

    target = ui.menu(menu, header="Choose target for %s" % desc,
      show_char=True)
    if target is None:
      return None
    else:
      targeted.append(src[target])

  return targeted


def descend_screen():
  # if you've already found the down stairs, go down immediately, otherwise
  # chance of encountering enemy
  ui.cwriteln("descending...")
  return back

def ascend_to_town_screen():
  ui.cwriteln("You carefully ascend from the depths, and return to town.")
  ui.wait_for_keypress()
  ui.cflush()
  return town_menu

def quit():
  okay = ui.confirm("~9Really quit (y/n)?~0 ")
  if okay:
    sys.exit(0)
  else:
    ui.cflush()
    return back

def party_death():
  sys.exit(0)

def level_up_creature(cr):
  from game import game

  import ui

  ui.cwriteln("%s gains a level!" % cr.name)

  old_max_hp = cr.max_hp
  old_max_mana = cr.max_mana
  old_stats = cr.stats.copy()

  cr.level_up()

  if cr.max_hp > old_max_hp:
    ui.cwriteln("~6-->~0 Max HP ~6+%d~0!" % (cr.max_hp - old_max_hp))
  if cr.max_mana > old_max_mana:
    ui.cwriteln("~6-->~0 Max mana ~6+%d~0!" % (cr.max_mana - old_max_mana))

  for stat in cr.stats:
    if cr.stats[stat] > old_stats[stat]:
      ui.cwriteln("~6-->~0 %s ~6+%d~0!" % (stat.short,
        cr.stats[stat] - old_stats[stat]))

def do_save():
  from game import game
  okay = ui.confirm("This will overwrite any existing saved game. Okay (y/n)? ")
  if okay:
    try:
      pickle.dump(game, open('lottof.dat', 'wb'))
      ui.cwriteln("Success!")
      ui.wait_for_keypress()
    except IOError:
      ui.cwriteln("Couldn't write to the file!")
  else:
    ui.cwriteln("Cancelled.")
  return

def restored_game():
  from game import game
  ui.cwriteln("Welcome back!")
  ui.wait_for_keypress()
  ui.cflush()
  ui.cflush()
  return temple_menu

def back():
  # special handling in loop below
  pass

def run():

  import game

  ui.init()
  ui.clear_screen()
  next = None

  if os.path.exists("lottof.dat"):
    try:
      game.game = pickle.load(open("lottof.dat", 'rb'))
    except IOError:
      sys.stderr.write("Couldn't load saved game!")
      sys.exit(-1)
    next = restored_game
  else:
    game.game = game.Game()
    game.game.init_test_game()
    next = start_game

  from game import game
  history = list()

  while True:
    next = next()
    if next is back:
      current = history.pop()
      next = history[-1]
    else:
      history.append(next)

if __name__ == '__main__':
  run()
