from random import choice, seed
import re
import sys

generic_rules = {
  'Phrase': ['Word', 'Word _ Word', 'Word _ Syl'],
  'Word': ['Syl', 'V Syl', 'Syl Syl', 'Syl Syl', 'V Syl Syl'],
  'Syl': [{'rule': 'C V', 'weight': 3}, {'rule': 'C V C#', 'weight': 8}],
  'C': ['p', 't', 't', 't', 'k', 'k', 'b', 'd', 'g', 's', 's', 's', 'sh', 'm', 'n', 'n', 'n', 'r', 'r', 'l', 'w', '\'', 'th'],
  'C#': ['n', 'n', 'n', 'm', 'r', 'r', 'l', 'th', 's'],
  'V': ['a', 'a', 'a', 'a', 'e', 'e', 'i', 'o', 'u', 'y', 'ai', 'au']
}

arabesque_rules = {
  'Phrase': ['LongWord', 'al- LongWord', 'ShortWord _ LongWord'],
  'ShortWord': ['C V Cvls', 'C V C Cvls', 'Cvls Cvls V C', 'V C V'],
  'LongWord': ['C V C V C', 'C V C V C', 'C V C V C V C'],
  'C': ['r', 'r', 'r', 'w', 'w', 'w', 'l', 'l', 'l', 'm', 'm', 'm',
    'n', 'n', 'n', 'b', 'b', 'f', 'f', 'q', 'q', '\'', '\'', 'd', 'd',
    's', 's', 'j', 'sh', 'k', 'h', 'z', 'kh', 't', 'gh', 'th', 'dh'],
  'Cvls': ['q', 'q', '\'', '\'', 's', 's', 'sh', 'k', 'f'],
  'V': ['i', 'i', 'ii', 'a', 'a', 'aa', 'u', 'uu']
}

japanman_rules = {
  'Phrase': ['LongWord', 'ShortWord _ LongWord',
    'LongWord _ ShortWord _ LongWord', 'LongWord _ no _ ShortWord'],
  'ShortWord': ['Syl'],
  'LongWord': ['Syl Syl', 'Syl Syl', 'Syl Syl Syl', 'Syl Syl Syl Syl'],
  'Syl': [{'rule': 'C Vaeo N?', 'weight': 8}, {'rule': 'Ci i N?', 'weight': 4},
    {'rule': 'Cu u N?', 'weight': 4}, 'Cy Vaeo N?'],
  'N?': ['', '', '', 'n'],
  'C': ['k', 's', 's', 't', 't', 'n', 'n', 'h', 'm', 'y', 'r', 'r', 'w', 'g',
    'z', 'd', 'b', 'p'],
  'Ci': ['k', 'sh', 'ch', 'n', 'h', 'm', 'r', 'g', 'j', 'b', 'p'],
  'Cu': ['k', 's', 'ts', 'n', 'f', 'm', 'y', 'r', 'g', 'z', 'b', 'p'],
  'Vaeo': ['a', 'e', 'o'],
  'Cy': ['ky', 'sh', 'ch', 'ny', 'hy', 'my', 'ry', 'gy', 'j', 'by', 'py']
}

rulesets = [generic_rules, arabesque_rules, japanman_rules]

class ContextFree(object):

  def __init__(self, rules, init_rule):
    self.rules = rules
    self.init_rule = init_rule
    self.accumulator = []

  def reset(self):
    self.accumulator = []

  def expand(self):
    self.reset()
    self.__expand(self.init_rule)

  def __expand(self, rule):
 
    expansion = []
    prob_table = []
    for index, elem in enumerate(self.rules[rule]):
      if type(elem) is dict:
        for i in range(elem['weight']):
          prob_table.append(index)
      else:
        prob_table.append(index)
  
    rulestr = ''
    elem = self.rules[rule][choice(prob_table)]
    if type(elem) is dict:
      rulestr = elem['rule']
    else:
      rulestr = elem
  
    for elemstr in rulestr.split():
      if elemstr in self.rules:
        self.accumulate(self.__expand(elemstr))
      else:
        self.accumulate(elemstr)

  def accumulate(self, instr):
    if instr is not None:
      self.accumulator.append(instr)

  def get(self):
    return ''.join(self.accumulator)
  
if __name__ == '__main__':

  for set in rulesets:
    cfree = ContextFree(set, 'Phrase') 
    for i in range(10):
      cfree.expand()
      print re.sub('_', ' ', cfree.get()).title()
    print ""
