from target_type import *
from item_type import *
from damage import *
from random import *
from stats import *
from effect import *
from roll import roll
from elements import *
from util import pos_int_gauss, first_upper

class Item(object):

  def __init__(self):

    self.name = 'default item'

    self.restrictions = dict() # restrictions on use
    self.target_type = TargetAny
    self.item_type = ItemType

    self.value = 1

    self.affinity = None # which element?
    self.use_effects = [] # effects that this object gives when used
    self.bonus = dict() # bonus for holding

    self.damage = None

  def __str__(self):
    ret = "%s (%s) Targets: %r\n" % (self.name, self.item_type.name,
      self.target_type)
    ret += "Restrictions: %s\n" % (self.restrictions)
    ret += "Elemental affinity: %s\n" % (self.affinity)
    ret += "Bonus: %s\n" % (self.bonus)
    ret += "When used:\n"
    for ef in self.use_effects:
      ret += "  %s\n" % ef
    ret += "Damage when wielded: %s\n" % (self.damage)
    ret += "Value: %d" % (self.value)
    return ret

  def list_str(self):
    ret = "%s " % (first_upper(self.name))
    if self.affinity is not None:
      ret += "(~6%s~0 %s) " % (self.affinity, self.item_type.name)
    else:
      ret += "(%s) " % (self.item_type.name)
    if self.damage is not None:
      ret += "Damage: ~6%s~0 " % self.damage
    if self.target_type is not None:
      ret += "Targets: ~6%s~0 " % self.target_type.short

    buffs = list()
    for s in stat_list:
      if s in self.bonus:
        buffs.append("%s:~6%+d~0" % (s.short, self.bonus[s]))
    if len(buffs) > 0:
      ret += "%s " % (' '.join(buffs))

    # print value    
    from game import game
    ret += "$%s:~6%d~0" % (game.currency_name_abbrev, self.value)

    restricts = list()
    for r in stat_list:
      if r in self.restrictions:
        if r in self.restrictions:
          restricts.append("%s>~6%d~0" % (r.short, self.restrictions[r]))
    if len(restricts) > 0:
      ret += "\nRequires: %s" % ' '.join(restricts)

    if len(self.use_effects) > 0:
      for ef in self.use_effects:
        ret += "\n"
        ret += ef.description

    return ret

def gen_armor(level=1):

  from game import game

  item = Item()
  item.item_type = choice(armor_types)
  item.name = "%s %s" % (game.get_random_general_name(), item.item_type.name)
  item.affinity = choice(game.element_matrix.elements)
  item.target_type = None
  item.value = pos_int_gauss(level*game.policy.level_price_multiplier)

  # guarantee defense buff
  item.bonus[Defense] = pos_int_gauss(level*2, level)

  # up to three buffs
  for i in range(randrange(1,4)): 
    random_stat = choice(stat_list)
    if random_stat not in item.bonus:
      next_val = pos_int_gauss(level, level)
      if next_val > 0:
        item.bonus[random_stat] = next_val

  return item

def gen_weapon(level=1):

  from game import game

  item = Item()
  item.item_type = WeaponItem
  item.target_type = choice([TargetOneFront, TargetAny])
  if item.target_type is TargetOneFront:
    item.name = "%s %s" % (game.get_random_general_name(), choice(['blade',
      'sword', 'axe', 'dagger', 'staff', 'hammer']))
  else:
    item.name = "%s %s" % (game.get_random_general_name(), choice(['bow',
      'spear', 'polearm', 'whip']))
  item.affinity = choice(game.element_matrix.elements)
  item.value = pos_int_gauss(level*game.policy.level_price_multiplier)

  item.damage = gen_damage(level)

  for i in range(level):
    if roll('d6') == 1:
      random_stat = choice([Strength, Agility, Magic])
      if random_stat not in item.restrictions:
        item.restrictions[random_stat] = 3 + pos_int_gauss(level/2.0, level/2.0)

  # for every level, 1 in 12 chance of stat buff
  for i in range(level):
    if roll('d12') == 1:
      random_stat = choice(stat_list)
      if random_stat not in item.bonus:
        next_val = pos_int_gauss(level, level)
        item.bonus[random_stat] = next_val

  return item

def gen_potion(level=1):

  from game import game
 
  item = Item()
  item.item_type = ItemType
  item.name = "%s %s" % (game.get_random_general_name(), choice(['potion',
    'scroll']))
  item.affinity = choice(game.element_matrix.elements)
  item.value = pos_int_gauss(level*game.policy.level_price_multiplier)

  effect_func = choice([gen_buff_effect, gen_buff_effect, gen_healing_effect,
    gen_healing_effect, gen_debuff_effect, gen_debuff_effect,
    gen_damage_effect, gen_poison_effect, gen_refresh_effect,
    gen_revive_effect])

  if effect_func in (gen_buff_effect, gen_healing_effect,
    gen_refresh_effect, gen_revive_effect):
    item.target_type = TargetFriendly
  else:
    item.target_type = choice([TargetAny, TargetAll])

  effect = effect_func(level) 
  item.use_effects.append(effect)

  return item

def gen_relic(level=1):

  """ relics: no use, just value """

  from game import game

  item = Item()
  item.item_type = RelicItem
  item.name = "the %s" % game.get_random_relic_name()
  item.value = level * game.policy.relic_value_level_multiplier
  item.value = int(gauss(item.value, item.value/4))
  item.target_type = None

  return item

if __name__ == '__main__':

  from game import game
  game.init_test_game()

  print "generating weapons"
  for i in range(1, 50, 2):
    print "at level %d" % i
    weapon = gen_weapon(i)
    print weapon

  relic = gen_relic()
  print relic

