from random import choice
import sys

class MarkovChain(object):

  def __init__(self, n=3, max=20):
    self.n = n
    self.max = max
    self.ch = dict()
    self.begin = list()

  def feed(self, line):
    self.begin.append(line[0:self.n])
    i = 0
    while i < len(line) - 1:
      frag = line[i:i+self.n]
      if frag not in self.ch:
        self.ch[frag] = list()
      self.ch[frag].append(line[i+self.n:i+self.n+1])
      i += 1

  def gen(self, disp=False, start_with=None):

    out = ''
    curr = ''
    if start_with is not None:
      curr = start_with
    else:
      curr = choice(self.begin)

    if disp:
      sys.stdout.write(curr)
    else:
      out += curr

    max = self.max
    while max:
      max -= 1
      if curr in self.ch:
        vals = self.ch[curr]
        if len(vals) == 0:
          break
        char = choice(vals)
        if disp:
          sys.stdout.write(char)
        else:
          out += char
        curr = (curr + char)[1:1+self.n]
      else:
        break

    if not disp:
      return out

if __name__ == '__main__':

  mark = MarkovChain(n=int(sys.argv[1]), max=25)
  lines = set()
  for line in sys.stdin:
    line = line.strip()
    mark.feed(line)
    lines.add(line)

  while True:
    next = mark.gen()
    if next not in lines:
      print next

