from creature import *
from item import *

class FrontPosition(object):
  pass

class BackPosition(object):
  pass

class TooManyMembersError(Exception):
  pass

class TooManyItemsError(Exception):
  pass

class Party(object):

  def __init__(self, name=None):
    from game import game
    self.front = list()
    self.back = list()
    self.inventory = list()
    self.sp = 0
    self.max_sp = game.policy.max_party_sp
    self.currency = 0
    self.name = name

  def all_members(self):
    members = list()
    members.extend(self.front)
    members.extend(self.back)
    return members

  def add_member(self, member, position=FrontPosition):
    # only allow so many creatures into party
    from game import game
    if len(self.all_members()) + 1 > game.policy.max_party_members:
      raise TooManyMembersError

    if position is FrontPosition:
      self.front.append(member)
    elif position is BackPosition:
      self.back.append(member)

  def assign_positions(self, mempos):
    for member, position in mempos:
      # skip if member already in that location
      if self.check_location(member, position):
        continue
      elif position is FrontPosition:
        self.back.remove(member)
        self.front.append(member)
      elif position is BackPosition:
        self.front.remove(member)
        self.back.append(member)

  def get_from_inventory(self, i):
    if isinstance(i, Item):
      self.inventory.remove(i)
      return i
    else:
      return self.inventory.pop(i)

  def add_to_inventory(self, item):
    from game import game
    if len(self.inventory) + 1 > game.policy.party_inventory_limit:
      raise TooManyItemsError
    else:
      self.inventory.append(item)
    self.inventory.sort(lambda a, b: cmp(a.name, b.name))

  def check_location(self, member, position):
    if position is FrontPosition:
      return member in self.front
    elif position is BackPosition:
      return member in self.back

  def remove_member(self, member):
    if member in self.front:
      self.front.remove(member)
    elif member in self.back:
      self.back.remove(member)

  def cull(self):
    """ remove dead members, move back row to front if necessary """
    defeated = list()
    for m in self.front[:]:
      if m.hp <= 0:
        self.front.remove(m)
        defeated.append(m)
    for m in self.back[:]:
      if m.hp <= 0:
        self.back.remove(m)
        defeated.append(m)
    if len(self.front) == 0 and len(self.back) > 0:
      self.front = self.back
      self.back = []
    return defeated

  def dead(self):
    living = [c for c in self.all_members() if c.hp > 0]
    if len(living) == 0:
      return True
    else:
      return False

  def __str__(self):
    ret = "Party: %s\n" % self.name
    ret += "Front row: %s\n" % ', '.join([m.name for m in self.front])
    ret += "Back row: %s\n" % ', '.join([m.name for m in self.back])
    ret += "Inventory: %s\n" % ', '.join([i.name for i in self.inventory])
    ret += "Currency: %d\n" % self.currency
    return ret

if __name__ == '__main__':

  from game import game
  game.init_test_game()

  creatures = list()
  party = Party()
  for i in range(6):
    try:
      new_creature = gen_creature(randrange(1,5))
      if random() > 0.5:
        party.add_member(new_creature, FrontPosition)
      else:
        party.add_member(new_creature, BackPosition)
    except TooManyMembersError, e:
      print "couldn't add %s -- too many members!" % new_creature.short_desc()

  print "members in front:" 
  for member in party.front:
    print member.short_desc()

  print "is %s in front?" % party.front[0].short_desc()
  print party.check_location(party.front[0], FrontPosition)
  
  print "is %s in back?" % party.back[0].short_desc()
  print party.check_location(party.back[0], BackPosition)

  print "members in back:"
  for member in party.back:
    print member.short_desc()

  print "adding stuff to inventory"
  for i in range(10):
    party.add_to_inventory(choice([gen_armor, gen_weapon, gen_potion])())

  for item in party.inventory:
    print item.name

  print "removing from inventory:"
  print party.get_from_inventory(0)
  
  for item in party.inventory:
    print item.name

